@@ -394,12 +394,12 @@ l7vs_conn_recv_client(struct l7vs_conn *conn)
 static int
 l7vs_conn_relay(struct l7vs_conn *conn, int is_client)
 {
-        char buf[L7VS_CONN_RELAY_BUFSIZE];
+        char buf[L7VS_CONN_RELAY_BUFSIZE + L7VS_PROTOMOD_MAX_ADD_BUFSIZE];
         char *rcvr, *sndr;
         int rfd, sfd;
-        int ret, len;
+        int ret, len, len_ret;
         int (*relayf)(struct l7vs_service *, struct l7vs_conn *,
-                       char *, size_t);
+                       char *, size_t *);
 
         relayf = NULL;
         if (is_client) {
@@ -428,10 +428,17 @@ l7vs_conn_relay(struct l7vs_conn *conn, int is_client)
         }
 
 	if (relayf != NULL) {
-		ret = (*relayf)(conn->srv, conn, buf, len);
+		len_ret = len;
+		ret = (*relayf)(conn->srv, conn, buf, &len_ret);
 		if (ret != 0) {
 			VANESSA_LOGGER_ERR("failed to analyze realserver data");
 		}
+		if (len > len_ret + L7VS_PROTOMOD_MAX_ADD_BUFSIZE){
+			VANESSA_LOGGER_ERR("bufsize too long modified by protomod ");
+			return -1;
+		} else {
+			len = len_ret;
+                }
 	}
 
         ret = send(sfd, buf, len, 0);
