@@ -60,6 +60,8 @@ struct l7vsadm_options {
 #define L7VSADM_OP_EDIT_VS      5
 #define L7VSADM_OP_EDIT_RS      6
 #define L7VSADM_OP_FLUSH_VS     7
+#define L7VSADM_OP_VERBOSE	8
+#define L7VSADM_OP_KEY		9
         int operation;
         struct sockaddr_in vaddr;
         struct sockaddr_in raddr;
@@ -118,6 +120,8 @@ usage(FILE *fp)
                 "  l7vsadm -d -t service-address -m proto-module [module-args]\n"
                 "          -r server-address\n"
                 "  l7vsadm -L|l [-n]\n"
+                "  l7vsadm -V\n"
+                "  l7vsadm -K\n"
                 "  l7vsadm -h\n\n");
         fprintf(fp,
                 "Commands:\n"
@@ -131,6 +135,8 @@ usage(FILE *fp)
                 "  --add-server      -a        add real server with options\n"
                 "  --edit-server     -e        edit real server with options\n"
                 "  --list            -L|l      list the table\n\n"
+                "  --verbose         -V        list the table in verbose format\n"
+                "  --key             -K        list the table in key setting format\n\n"
                 );
         fprintf(fp,
                 "Options:\n"
@@ -144,11 +150,13 @@ usage(FILE *fp)
 static int
 parse_args(struct l7vsadm_options *opt, int argc, char *argv[])
 {
-#define OPTION_STRING   "-lADECadenr:m:t:s:w:"
+#define OPTION_STRING   "-lVKADECadenr:m:t:s:w:"
 
         static struct option options[] = {
                 /* operation options (toplevel) */
                 {"list",                no_argument,            NULL, 'l'},
+                {"verbose",		no_argument,            NULL, 'V'},
+		{"key",			no_argument,		NULL, 'K'},
                 {"add-service",         no_argument,            NULL, 'A'},
                 {"delete-service",      no_argument,            NULL, 'D'},
                 {"edit-service",        no_argument,            NULL, 'E'},
@@ -191,6 +199,12 @@ do {                                                                    \
                 case 'l':
                         SET_OP(opt, L7VSADM_OP_LIST);
                         break;
+                case 'V':
+                        SET_OP(opt, L7VSADM_OP_VERBOSE);
+                        break;
+		case 'K':
+			SET_OP(opt, L7VSADM_OP_KEY);
+			break;
                 case 'A':
                         SET_OP(opt, L7VSADM_OP_ADD_VS);
                         break;
@@ -598,9 +612,11 @@ retry:
                 return -1;
         }
 
-        printf("Layer-7 Virtual Server version %s\n", l7vs_version_string);
-        printf("Prot LocalAddress:Port Scheduler ProtoMod\n"
-               "  -> RemoteAddress:Port           Forward Weight ActiveConn InactConn\n");
+	if (bufsize == L7VSADM_LIST_VS_BUFSIZE) {
+        	printf("Layer-7 Virtual Server version %s\n", l7vs_version_string);
+        	printf("Prot LocalAddress:Port Scheduler ProtoMod\n"
+        	       "  -> RemoteAddress:Port           Forward Weight ActiveConn InactConn\n");
+	}
 
         req.cmd = L7VS_CONFIG_LIST_VS;
         ret = send_request(s, &req, sizeof(req));
@@ -612,7 +628,7 @@ retry:
         iov[0].iov_base = &rsp;
         iov[0].iov_len = sizeof(rsp);
         iov[1].iov_base = buf;
-        iov[1].iov_len = L7VSADM_LIST_VS_BUFSIZE;
+        iov[1].iov_len = bufsize;
 
         ret = recv_response(s, iov, sizeof(iov) / sizeof(iov[0]));
         if (ret < 0) {
@@ -639,8 +655,19 @@ retry:
                 if (srvname == NULL) {
                         break;
                 }
-                printf("TCP %s %s %s\n",
+
+		/* If it ie verbose, display more details else simple details */
+		if (opt->operation == L7VSADM_OP_VERBOSE) {
+                	printf("TCP %s %s %s %d %s\n",
+                       srvname, sarg->protomod, sarg->schedmod, sarg->reschedule, sarg->protomod_opt_string);
+		} else if (opt->operation == L7VSADM_OP_KEY) {
+                	printf("TCP %s %s %s %d %s\n",
+                       srvname, sarg->protomod, sarg->schedmod, sarg->reschedule, sarg->protomod_key_string);
+		} else {
+                	printf("TCP %s %s %s\n",
                        srvname, sarg->protomod, sarg->schedmod);
+		}
+
                 free(srvname);
                 list_rs(s, sarg, opt->isnumeric);
 
@@ -1019,6 +1046,8 @@ main(int argc, char *argv[])
                 {L7VSADM_OP_DEL_RS,     operate_rs},
                 {L7VSADM_OP_EDIT_RS,    operate_rs},
                 {L7VSADM_OP_FLUSH_VS,   flush_vs},
+                {L7VSADM_OP_VERBOSE,	list_vs},
+                {L7VSADM_OP_KEY,	list_vs},
                 {-1, NULL}
         };
 
